#include <stdio.h>
#include <string.h>
#include "error.h"
#include "config.h"
#include "cdbslib.h"
#include "reffile.h"
#include "loadfile.h"
#include "uniqname.h"
#include "cdbserror.h"

//====================================================================
// UNIQNAME -- Rename a file according to CDBS conventions
//
// i: cdbscatalog  lookup table of reference file types
// i: oldname	file to be renamed
// u: newlist	new names associated with old file names
//
// History
// -------
// 06/12/02  45993  mss  add error case when no change in name occurs
//====================================================================

int uniqname (table *cdbscatalog, char *oldname, alist *newlist)
{
    double	mjd;
    FILE	*fd;
    int		matching_row, irow, status;
    char	altname[SZ_FNAME+1], childname[SZ_FNAME+1];
    char	newname[SZ_FNAME+1], date[SZ_STRING+1], history[SZ_LINE+1];

    // Check to see if file has already been renamed 
    if (getlistname (newlist, oldname, altname, SZ_FNAME) == OK) {
        if ((fd = fopen (altname, "r")) != NULL) {
	    fclose (fd);
	    return (OK);
	}
    }

    // Read reference file type, file type, and instrument from cdbscatalog
    if ((matching_row = find_lookup (cdbscatalog, oldname)) == ERR) {
	cdbs_warn (NOLOOK, "%s", oldname);
	return (ERR);
    }
    char *instrument = get_tab_cell (cdbscatalog, "instrument", matching_row);
    char *reftype    = get_tab_cell (cdbscatalog, "reftype", matching_row);
    char *filetype   = get_tab_cell (cdbscatalog, "filetype", matching_row);

    // determine file type of the reference file (image, table, load)
    char actual_file_type[20];
    type_reffile (oldname, actual_file_type);
    int file_type=UNRECOGNIZED;
    if (strcmp(actual_file_type,"table")==0) {
       file_type=TABLE;
    }
    if (strcmp(actual_file_type,"image")==0) {
       file_type=IMAGE;
    }
    if (strcmp(actual_file_type,"load")==0) {
       file_type=LOADFILE;
    }

    // Call uniqname recursively on any files mentioned in this file 
    reffile *ref = NULL;
    status = OK;
    switch (file_type) {

    // case IMAGE: no action

    case TABLE:
        ref = open_reffile_body (oldname, filetype, "r+");
        if (ref == NULL) {
            cdbs_warn (NOTOPEN, "%s", oldname);
	    return (ERR);
        }
	if (get_body_reffile (ref, "HEADER_FILE", 0, 
			      childname, SZ_FNAME) == OK) {

	    if (renamable (newlist, ref) == NO) {
	        status = ERR;
		break;
	    }

	    for (irow = 0; irow < nrow_reffile (ref); irow ++) {
	        get_body_reffile (ref, "HEADER_FILE", irow, 
				  childname, SZ_FNAME);
		strlcase (childname);

		if (uniqname (cdbscatalog, childname, newlist) == ERR) {
		    status = ERR;
		    break;
		}
	    }
	}
        close_reffile(ref);
        break;

    case LOADFILE:
        ref = open_reffile_hdr (oldname, filetype, "r+");
        if (ref == NULL) {
            cdbs_warn (NOTOPEN, "%s", oldname);
	    return (ERR);
        }
	if (get_hdr_reffile (ref, "FILE_NAME", childname, SZ_FNAME) == OK) {
	    if (uniqname (cdbscatalog, childname, newlist) == ERR)
		status = ERR;
	}
        close_reffile(ref);
        break;
    }   // end switch

    if (status == ERR) {
        cdbs_warn (NOCHILD, "%s", oldname);
	return (ERR);
    }

    // open for HEADER access and generate a new filename
    //
    ref = open_reffile_hdr (oldname, filetype, "r+");
    if (ref == NULL) {
        cdbs_warn (NOTOPEN, "%s", oldname);
	return (ERR);
    }
    if (getuniqname (ref, reftype, filetype, instrument, 
		     newlist, newname, SZ_FNAME) == ERR) {
	cdbs_warn (NONUNIQ, "%s", oldname);
	close_reffile (ref);
	return (ERR);
    }

    if (strcmp(oldname, newname) == 0) {
        fprintf(stderr,"ERROR: Old and new names MATCH (%s)\n", oldname);
        return (ERR);
    }

    // use HEADER access and change names embedded in file; add to arglist 
    if (setnames (&ref, newlist, newname, file_type) == ERR) {
	cdbs_warn (NOSETNAM, "%s", oldname);
	close_reffile (ref);
	return (ERR);
    }

    // Add a history record to the reference file if it is not a loadfile 
    if (strcmp(actual_file_type,"load")!=0) {
	mjd = get_now ();
	put_short_date (mjd, date, SZ_STRING);

	sprintf (history, "%s renamed to %s on %s", oldname, newname, date);
	put_hist_reffile (ref, "HISTORY", history);
    }

    // Rename the file itself 
    close_reffile (ref);

    if (rename_reffile (oldname, newname, filetype) == ERR) {
	cdbs_warn (RENAME, "%s", oldname);
	return (ERR);
    }

    cdbs_info (NEWNAME, "%s -> %s", oldname, newname);
    return (OK);
}
